@@ -12,7 +12,9 @@
  * Lesser General Public License for more details.
  *
  * You should have received a copy of the GNU Lesser General Public
- * License along with this library. If not, see <http://www.gnu.org/licenses/>.
+ * License along with this library; if not, write to the
+ * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
+ * Boston, MA 02111-1307, USA.
  */
 
 /*
@@ -26,8 +28,8 @@
 #include "gtkcheckmenuitem.h"
 #include "gtkmenuitemprivate.h"
 #include "gtkaccellabel.h"
-#include "deprecated/gtkactivatable.h"
-#include "deprecated/gtktoggleaction.h"
+#include "gtkactivatable.h"
+#include "gtktoggleaction.h"
 #include "gtkmarshalers.h"
 #include "gtkprivate.h"
 #include "gtkintl.h"
@@ -53,6 +55,7 @@
 struct _GtkCheckMenuItemPrivate
 {
   guint active             : 1;
+  guint always_show_toggle : 1;
   guint draw_as_radio      : 1;
   guint inconsistent       : 1;
 };
@@ -95,12 +98,9 @@ static void gtk_check_menu_item_sync_action_properties     (GtkActivatable
 static GtkActivatableIface *parent_activatable_iface;
 static guint                check_menu_item_signals[LAST_SIGNAL] = { 0 };
 
-G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
 G_DEFINE_TYPE_WITH_CODE (GtkCheckMenuItem, gtk_check_menu_item, GTK_TYPE_MENU_ITEM,
-                         G_ADD_PRIVATE (GtkCheckMenuItem)
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ACTIVATABLE,
                                                 gtk_check_menu_item_activatable_interface_init))
-G_GNUC_END_IGNORE_DEPRECATIONS;
 
 static void
 gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
@@ -177,6 +177,8 @@ gtk_check_menu_item_class_init (GtkCheckMenuItemClass *klass)
                   NULL, NULL,
                   _gtk_marshal_VOID__VOID,
                   G_TYPE_NONE, 0);
+
+  g_type_class_add_private (klass, sizeof (GtkCheckMenuItemPrivate));
 }
 
 static void 
@@ -193,14 +195,11 @@ gtk_check_menu_item_update (GtkActivatable *activatable,
                             const gchar    *property_name)
 {
   GtkCheckMenuItem *check_menu_item;
-  gboolean use_action_appearance;
 
   check_menu_item = GTK_CHECK_MENU_ITEM (activatable);
 
   parent_activatable_iface->update (activatable, action, property_name);
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-
   if (strcmp (property_name, "active") == 0)
     {
       gtk_action_block_activate (action);
@@ -208,19 +207,12 @@ gtk_check_menu_item_update (GtkActivatable *activatable,
       gtk_action_unblock_activate (action);
     }
 
-  use_action_appearance = gtk_activatable_get_use_action_appearance (activatable);
-  G_GNUC_END_IGNORE_DEPRECATIONS;
-
-  if (!use_action_appearance)
+  if (!gtk_activatable_get_use_action_appearance (activatable))
     return;
 
   if (strcmp (property_name, "draw-as-radio") == 0)
-    {
-      G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-      gtk_check_menu_item_set_draw_as_radio (check_menu_item,
-                                             gtk_toggle_action_get_draw_as_radio (GTK_TOGGLE_ACTION (action)));
-      G_GNUC_END_IGNORE_DEPRECATIONS;
-    }
+    gtk_check_menu_item_set_draw_as_radio (check_menu_item,
+                                           gtk_toggle_action_get_draw_as_radio (GTK_TOGGLE_ACTION (action)));
 }
 
 static void
@@ -228,35 +220,23 @@ gtk_check_menu_item_sync_action_properties (GtkActivatable *activatable,
                                             GtkAction      *action)
 {
   GtkCheckMenuItem *check_menu_item;
-  gboolean use_action_appearance;
-  gboolean is_toggle_action;
 
   check_menu_item = GTK_CHECK_MENU_ITEM (activatable);
 
   parent_activatable_iface->sync_action_properties (activatable, action);
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
-  is_toggle_action = GTK_IS_TOGGLE_ACTION (action);
-  G_GNUC_END_IGNORE_DEPRECATIONS;
-
-  if (!is_toggle_action)
+  if (!GTK_IS_TOGGLE_ACTION (action))
     return;
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_action_block_activate (action);
-
   gtk_check_menu_item_set_active (check_menu_item, gtk_toggle_action_get_active (GTK_TOGGLE_ACTION (action)));
   gtk_action_unblock_activate (action);
-  use_action_appearance = gtk_activatable_get_use_action_appearance (activatable);
-  G_GNUC_END_IGNORE_DEPRECATIONS;
-
-  if (!use_action_appearance)
+  
+  if (!gtk_activatable_get_use_action_appearance (activatable))
     return;
 
-  G_GNUC_BEGIN_IGNORE_DEPRECATIONS;
   gtk_check_menu_item_set_draw_as_radio (check_menu_item,
                                          gtk_toggle_action_get_draw_as_radio (GTK_TOGGLE_ACTION (action)));
-  G_GNUC_END_IGNORE_DEPRECATIONS;
 }
 
 /**
@@ -481,8 +461,15 @@ gtk_check_menu_item_get_draw_as_radio (GtkCheckMenuItem *check_menu_item)
 static void
 gtk_check_menu_item_init (GtkCheckMenuItem *check_menu_item)
 {
-  check_menu_item->priv = gtk_check_menu_item_get_instance_private (check_menu_item);
-  check_menu_item->priv->active = FALSE;
+  GtkCheckMenuItemPrivate *priv;
+
+  check_menu_item->priv = G_TYPE_INSTANCE_GET_PRIVATE (check_menu_item,
+                                                       GTK_TYPE_CHECK_MENU_ITEM,
+                                                       GtkCheckMenuItemPrivate);
+  priv = check_menu_item->priv; 
+
+  priv->active = FALSE;
+  priv->always_show_toggle = TRUE;
 }
 
 static gint
@@ -571,29 +558,37 @@ gtk_real_check_menu_item_draw_indicator (GtkCheckMenuItem *check_menu_item,
 
       y = (allocation.height - indicator_size) / 2;
 
-      gtk_style_context_save (context);
-
-      if (priv->inconsistent)
-        state |= GTK_STATE_FLAG_INCONSISTENT;
-      else if (priv->active)
-        state |= GTK_STATE_FLAG_ACTIVE;
-
-      gtk_style_context_set_state (context, state);
-
-      if (priv->draw_as_radio)
+      if (priv->active ||
+          priv->always_show_toggle ||
+          (gtk_widget_get_state_flags (widget) & GTK_STATE_FLAG_PRELIGHT))
         {
-          gtk_style_context_add_class (context, GTK_STYLE_CLASS_RADIO);
-          gtk_render_option (context, cr, x, y,
-                             indicator_size, indicator_size);
-        }
-      else
-        {
-          gtk_style_context_add_class (context, GTK_STYLE_CLASS_CHECK);
-          gtk_render_check (context, cr, x, y,
-                            indicator_size, indicator_size);
-        }
+          gtk_style_context_save (context);
 
-      gtk_style_context_restore (context);
+          if (priv->inconsistent)
+            state |= GTK_STATE_FLAG_INCONSISTENT;
+          else if (priv->active)
+            state |= GTK_STATE_FLAG_ACTIVE;
+
+          if (!gtk_widget_is_sensitive (widget))
+            state |= GTK_STATE_FLAG_INSENSITIVE;
+
+          gtk_style_context_set_state (context, state);
+
+          if (priv->draw_as_radio)
+            {
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_RADIO);
+              gtk_render_option (context, cr, x, y,
+                                 indicator_size, indicator_size);
+            }
+          else
+            {
+              gtk_style_context_add_class (context, GTK_STYLE_CLASS_CHECK);
+              gtk_render_check (context, cr, x, y,
+                                indicator_size, indicator_size);
+            }
+
+          gtk_style_context_restore (context);
+        }
     }
 }
 
